pub fn PageEntryMixin() type {
    return struct {
        const present_bit: u64 = 1 << 0;
        const read_write_bit: u64 = 1 << 1;
        const user_supervisor_bit: u64 = 1 << 2;
        const ps_bit: u64 = 1 << 7;
        const no_execute_bit: u64 = 1 << 63;

        pub inline fn no_execute(self: var) bool {
            return self.value & noExecute_bit != 0;
        }

        pub inline fn no_executeSet(self: var, is_no_execute: bool) void {
            if (is_no_execute) {
                self.value |= no_execute_bit;
            } else {
                self.value &= ~no_execute_bit;
            }
        }

        pub inline fn present(self: var) bool {
            return self.value & present_bit != 0;
        }

        pub inline fn presentSet(self: var, is_present: bool) void {
            if (is_present) {
                self.value |= present_bit;
            } else {
                self.value &= ~present_bit;
            }
        }

        pub inline fn ps(self: var) bool {
            return self.value & ps_bit != 0;
        }

        pub inline fn psSet(self: var, active: bool) void {
            if (active) {
                self.value |= ps_bit;
            } else {
                self.value &= ~ps_bit;
            }
        }

        pub inline fn read_write(self: var) void {
            return self.value & read_write_bit != 0;
        }

        pub inline fn read_writeSet(self: var, allow_write: bool) void {
            if (allow_write) {
                self.value |= read_write_bit;
            } else {
                self.value &= ~read_write_bit;
            }
        }

        pub inline fn user_supervisor(self: var) void {
            return self.value & user_supervisor_bit != 0;
        }

        pub inline fn user_supervisorSet(self: var, allow_user: bool) void {
            if (allow_user) {
                self.value |= user_supervisor_bit;
            } else {
                self.value &= ~user_supervisor_bit;
            }
        }
    };
}
